# Preproc data. The data are saved in a tabular format in the ./Data subdirectory. 
# Here, they are reorganized for use in a random effect model.

library(tidyr)
library(dplyr)

get_activity <- function(phenotype) {
  ifelse (phenotype == "PM", -2,
          ifelse(phenotype == "IM", -1, 
                 ifelse(phenotype == "EM", 0, 
                        ifelse(phenotype == "RM", 0.8, 1.6))))
}

studies <- read.table("./Data/CYP2C19Data.txt", header=T)
tmp1 <- studies %>% select(-starts_with("Size")) %>% 
                 pivot_longer(cols = starts_with("Adjustment"), 
                     values_to = "Adjustment",
                     names_to = "Phenotype", names_prefix = "Adjustment_")
tmp2 <- studies %>% select(-starts_with("Adjustment")) %>% 
                 pivot_longer(cols = starts_with("Size"), values_to = "Size", 
                    names_to = "Phenotype", names_prefix = "Size_")
cyp <- inner_join(tmp1, select(tmp2, StudyID, Phenotype, Size), 
                  by = c("StudyID", "Phenotype")) %>% drop_na() %>%
        mutate(Activity = get_activity(Phenotype), 
               Selection = Status == "OK" | Status == "NoSubstrate")

#recode rare methods
cyp$Parameter[cyp$Parameter == "1-Cl"] <- "Cl"
cyp$Parameter[cyp$Parameter == "Css-D"] <- "Css"
cyp$Parameter[cyp$Parameter == "MR"] <- "Css"
print("Recoded methods: Css-D -> Css")
print("                 MR -> Css")
print("                 1-Cl -> Cl")

cyp <- select(cyp, -IM_11)
print("allelic determnation: covariates IM11 dropped as there are no cases")

cyp$IM_17[cyp$Phenotype != "IM"] <- 0
cyp$RM_1717[cyp$Phenotype != "RM"] <- 0
cyp$RM_11[cyp$Phenotype != "RM"] <- 0
print("allelic determination: IM and RM limited to own phenotype")

#code interactions
cyp <- within(cyp, {
  SDxActivity <- Activity * as.numeric(Dosage == "SD")
  AUCxActivity <- Activity * as.numeric(Parameter == "Auc")
  CssxActivity <- Activity * as.numeric(Parameter == "Css")
  EM_17xAct <- Activity * as.numeric(EM_17)  # in the EM there are also *17
  EM_17xAct17 <- EM_17xAct * as.numeric(Phenotype == "RM" | Phenotype == "UM")
  EM_17xAct2  <- EM_17xAct * as.numeric(Phenotype == "IM" | Phenotype == "PM")
  EM_2xAct <- Activity * as.numeric(EM_2)    # in the EM there are also *2
  EM_2xAct17 <- EM_2xAct * as.numeric(Phenotype == "RM" | Phenotype == "UM")
  EM_2xAct2  <- EM_2xAct * as.numeric(Phenotype == "IM" | Phenotype == "PM")
  IM_17xAct <- Activity * as.numeric(IM_17)  # in the IM (*2) there are also *17
  IM_22xAct <- Activity * as.numeric(IM_22)  # in the IM (*2) there are also *2*2 (gepoolt, *one* study pooled IM and PM)
  # IM_11: no case
  RM_1717xAct <- Activity * as.numeric(RM_1717) # in the RM (*17) there are also *17*17 (RM and UM gepoolt)
  RM_11xAct <- Activity * as.numeric(RM_11) }# in the RM (*17) there are also also *1 (EMs) -- same as EM_17
)

phen_levels <- c("PM", "IM", "EM", "RM", "UM")
print("dataframe cyp: all records with activity levels")

rm(tmp1)
rm(tmp2)

# get rid of EMs, as we do not use them in models (dataframe cypsel)
cypsel <- filter(cyp, Selection == TRUE, Phenotype != "EM")
cypsel$weights <- cypsel$Size / sum(cypsel$Size) * nrow(cypsel)
print("dataframe cypsel: only admissible records, no EMs")

#utility function to plot data from fit
extract_phenotype <- function(fitb, pht) {
  #select hs variables
  sample <- select(as.data.frame(fitb), matches("b\\[Activity"))
  if (0 == ncol(sample))
    sample <- select(as.data.frame(fitb), matches("^beta\\["))

  #(get rid of coefficients modelled as fixed)
  nfixed <- dim(select(as.data.frame(fitb), matches("^b\\[\\d+\\]$")))[2]
  sample <- sample[,1:(ncol(sample)-nfixed)]
  if (ncol(sample) > 18) sample <- sample[,1:18]
  
  #compute effect
  sample <- get_activity(pht) * sample + 100
  
  #rename sample
  names(sample) <- mednames
  sample
}

library(ggplot2)
#utility function to plot intervals
plot_intervals <- function(fitobj, phenotype) {
  p <- mcmc_intervals(extract_phenotype(fitobj, phenotype), prob_outer = 0.9, prob = 0.5) + 
    xlab(paste("Est. adjusted dose", phenotype, "(90%/50% intervals)")) + 
    vline_at(100) + vline_at(80, linetype = 2) + vline_at(120, linetype = 2) + 
    vline_at(50, linetype = 3)
  p
}

library(stringi)
plot_multiple_intervals <- function(models, mnames, phentype) {
  # example:
  #  phentype = "IM"
  #  models <- c(fixfit, hsfit, rhsfit)
  #  mnames <- c("fixed", "horseshoe", "reghorsesh")
  
  data <- extract_phenotype(models[[1]], phentype)
  names(data) <- paste(names(data), mnames[1], sep = "_")
  for (i in 2:length(models)) {
    buff <- extract_phenotype(models[[i]], phentype)
    names(buff) <- paste(names(buff), mnames[i], sep = "_")
    data <- cbind(data, buff)
  }
  data <- data %>% select(order(names(.), decreasing=T))
  
  p <- mcmc_intervals(data)
  dp <- p$data
  dp <- mutate(dp, model = stri_extract_last(parameter,regex = "[A-Za-z]+"))
  dp <- mutate(dp, substance = stri_extract_first(parameter, regex = "[A-Za-z]+"))
  nmodels <- length(unique(dp$model))
  dp$poshead <- as.integer(as.factor(dp$substance)) * nmodels * 3
  dp$position <-  dp$poshead - as.integer(as.factor(dp$model)) * 2
  dp$poslabel <- dp$poshead - nmodels / 2 * 2
  
  ggplot(dp, aes(color = model, y = position, yend = position)) + 
    geom_segment(aes(x = ll, xend = hh)) +
    geom_segment(aes(x = l, xend = h), size = 1.5) + 
    geom_point(aes(x = m, fill = model), color = "black", size = 2, shape = 21) +
    vline_at(100) + 
    scale_y_continuous(name = "substance", breaks = dp$poslabel, labels = dp$substance) + 
    scale_color_viridis_d(option = "E", begin = 0.2, end = 0.9) + 
    scale_fill_viridis_d(option = "E", begin = 0.1, end = 0.8) +
    #scale_color_brewer(direction = -1, type = "qual") + 
    #scale_fill_brewer(direction = -1) +
    xlab("Estimated adjusted dose") +
    theme_minimal(base_size = 14)
}

#utility function to plot adjustments by substance
theme_custom <- theme_minimal(base_size = 16) %+replace% theme(panel.grid = element_blank())
plot_adj <- function(fit) {
  cypselex <- filter(cyp, Selection == TRUE)
  if (0 == ncol(select(as.data.frame(fit), matches("^fadjust")))) {
    f <- as.data.frame(fit) %>% select(matches("^f")) %>% map_df(median) %>% as.matrix() %>% t()
  } else {
    f <- as.data.frame(fit) %>% select(matches("^fadjust")) %>% map_df(median) %>% as.matrix() %>% t()
  }
  cypsel$fitted <- f[,1]
  cypselex <- left_join(cypselex, select(cypsel, StudyID, Phenotype, fitted), by = c("StudyID", "Phenotype"))
  cypselex$fitted <- replace_na(cypselex$fitted, 0)
  cypselmin <- filter(cypselex, Phenotype != "EM")
  p <- ggplot(cypselex, aes(x = Activity)) + 
    vline_at(0, color="gray") + hline_at(100, color="gray") +
    geom_boxplot(data = cypselmin, aes(group = Activity, y = Adjustment), outlier.size = 0) +
    geom_point(data = cypselmin, aes(y = Adjustment, size=Size), color = "gray", shape=1) + 
    geom_line(aes(y = 100 + fitted), color = "#1030A0", size = 1) + 
    scale_x_continuous(labels = c("PM", "IM", "", "RM", "UM")) + 
    scale_y_continuous("Adjusted dose (in % of label)") + 
    scale_size_continuous("Sample size") + 
    theme_custom + facet_wrap(.~ Substance) + 
    theme(aspect.ratio = 1.1, legend.position = "right", 
          legend.text = element_text(size = 12)) + 
    xlab("Allelic phenotype")
  p
}

#utility function to plot fits
library(purrr)
plot_mikado <- function(fit) {
  cypselex <- filter(cyp, Selection == TRUE)
  if (0 == ncol(select(as.data.frame(fit), matches("^fadjust")))) {
    f <- as.data.frame(fit) %>% select(matches("^f")) %>% map_df(median) %>% as.matrix() %>% t()
  } else {
    f <- as.data.frame(fit) %>% select(matches("^fadjust")) %>% map_df(median) %>% as.matrix() %>% t()
  }
  cypsel$fitted <- f[,1]
  cypselex <- left_join(cypselex, select(cypsel, StudyID, Phenotype, fitted), by = c("StudyID", "Phenotype"))
  cypselex$fitted <- replace_na(cypselex$fitted, 0)
  #dataset without EMs
  cypselmin <- filter(cypselex, Phenotype != "EM")
  #dataset where we have estimates for all allelic phenotypes
  substances <- cypselex %>% select(StudyID, Substance, Phenotype) %>% 
    group_by(Substance, Phenotype) %>% 
    summarize(count = n_distinct(Phenotype)) %>% 
    summarize(countphens = sum(count)) %>% filter(countphens > 3)
  cypx <- semi_join(cypselex, substances, by = "Substance")
  
  p <- ggplot(cypselex, aes(x = Activity)) + 
    geom_boxplot(data = cypselmin, aes(group = Activity, y = Adjustment), outlier.size = 0) +
    geom_line(aes(y = 100 + fitted, color = Substance), alpha = 0.5, linetype = 3) + 
    geom_line(data = cypx, aes(y = 100 + fitted, color = Substance)) +
    geom_point(data = cypselmin, aes(y = Adjustment, color=Substance, size=Size), shape=1) + 
    scale_y_continuous("Adjusted dose (in % of label)") + 
    scale_size_continuous("Sample size") + 
    theme_classic(base_size = 16)
  p
}

#utility function to plot fits (frequentist fits)
boxpl_cyp_ <- function(fit) {
  p <- ggplot(cypselex, aes(x = Activity)) + 
    geom_boxplot(aes(group = Activity, y = Adjustment), outlier.size = 0) +
    geom_line(data = cypsel, aes(y = 100 + fitted(fit), group = Substance, color = Substance)) + 
    geom_point(aes(x=Activity, y=Adjustment, color=Substance, size=Size), shape=1) + 
    scale_y_continuous("Adjusted dose (in % of label)") + 
    scale_size_continuous("Sample size") + 
    theme_classic(base_size = 16)
  p
}

#utility function to plot residuals
plot_mikado_res <- function(fit) {
  cypselex <- filter(cyp, Selection == TRUE)
  f <- as.data.frame(fit) %>% select(matches("^f\\[")) %>% map_df(median) %>% as.matrix() %>% t()
  weights <- 1/(as.data.frame(fit) %>% select(matches("^disp")) %>% map_df(median) %>% as.matrix() %>% t())
  p <- ggplot(cypsel, aes(x = Activity, y = (Adjustment - f - 100))) +
    geom_boxplot(aes(group = Activity), outlier.size = 0) +
    geom_point(aes(color=Substance, size=Size), shape=1) + 
    scale_y_continuous("Residuals") + 
    scale_size_continuous("Sample size") + 
    theme_classic(base_size = 16)
  p
}
